/*----------------------------------------------------------------------------

   libtunepimp -- The MusicBrainz tagging library.  
                  Let a thousand taggers bloom!
   
   Copyright (C) Robert Kaye 2003
   
   This file is part of libtunepimp.

   libtunepimp is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   libtunepimp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with libtunepimp; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   $Id: tp_c.h,v 1.8 2003/07/25 23:23:21 robert Exp $

----------------------------------------------------------------------------*/
#ifndef _TP_C_H_
#define _TP_C_H_

#ifdef __cplusplus
extern "C"
{
#endif

#include "defs.h"

/**
 *  * Basic C abstraction for the TunePimp Object 
 */
typedef void *tunepimp_t;
typedef void *track_t;
typedef void *result_t;

typedef void (*tp_notify_callback)(TPCallbackEnum, int fileId);
typedef void (*tp_status_callback)(const char *status);

typedef struct _metadata_t
{
    char         *artist;
    char         *sortName;
    char         *album;
    char         *track;
    int           trackNum;
    int           variousArtist;
    char         *artistId;   
    char         *albumId;   
    char         *trackId;
    char         *fileTrm;
    char         *albumArtistId;
    unsigned long duration;
    TPAlbumType   albumType;

    // This is only used in case of TRM collision
    int           numTRMIds;
} metadata_t;

typedef struct _artistresult_t
{
    int   relevance;
    char *name, *sortName;
    char *id;
} artistresult_t;

typedef struct _albumresult_t
{
    int             relevance;
    char           *name;
    char           *id;
    int             numTracks, numCDIndexIds;
    int             isVA;
    TPAlbumStatus   status;
    TPAlbumType     type;
    artistresult_t *artist;
} albumresult_t;

typedef struct _albumtrackresult_t
{
    int             relevance;
    char           *name;
    char           *id;
    int             numTRMIds, trackNum;
    unsigned long   duration;
    artistresult_t *artist;
    albumresult_t  *album;
} albumtrackresult_t;
/* --------------------------------------------------------------------------
 * Main TunePimp Interface
 * --------------------------------------------------------------------------*/

/**
 * Create a new handle (a C abstraction) to the TunePimp object.
 * Call tp_Delete() when done with the handle.
 * @see tp_Delete()
 * @return the tunepimp_t type is used in subsequent tunepimp functions.
 */
tunepimp_t tp_New           (const char *appName, const char *appVersion);

/**
 * The destructor for the TunePimp class.
 * @see tp_New()
 * @param o the tunepimp_t object to delete
 */
void      tp_Delete         (tunepimp_t o);

/**
 * Get the version number of this library
 * @param o the tunepimp_t object returned from tp_New
 * @param major an int pointer that will receive the major number of the version
 * @param minor an int pointer that will receive the minor number
 * @param rev   an int pointer that will receive the rev number
 */
void      tp_GetVersion     (tunepimp_t o, int *major, int *minor, int *rev);

/**
 * This function must be called if you want to submit data to the server
 * If you're looking up data from the server, you do not need to call tp_SetUserInfo.
 * @param o the tunepimp_t object returned from tp_New()
 * @param userName the name of the user
 * @param password the plaintext password of the user.
 */
void       tp_SetUserInfo    (tunepimp_t o, const char *userName, 
                                            const char *password);

/**
 * Use this function to set the encoding for the data returned by the tunepimp 
 * library to UTF-8.
 * @see 
 * @param o the tunepimp_t object returned from tp_New()
 * @param useUTF8 if set to a non-zero value, UTF-8 will be used.
 */
void      tp_UseUTF8      (tunepimp_t o, int useUTF8);

/**
 * Set the name and the port of the MusicBrainz server to use. If this
 * function is not called, the default www.musicbrainz.org server on port
 * 80 will be used.
 * @see tp_SetProxy
 * @param o the tunepimp_t object returned from tp_New
 * @param serverAddr the name of the musicbrainz server to use 
 *                   e.g. www.musicbrainz.org
 * @param serverPort the port number to use. e.g. 80
 */
void       tp_SetServer      (tunepimp_t o, const char *serverAddr, 
                              short serverPort);

/**
 * Set the name of the HTTP Proxy to use. This function must be called anytime
 * the client library must communicate via a proxy firewall. 
 * @see  tp_SetServer
 * @param o the tunepimp_t object returned from tp_New()
 * @param serverAddr the name of the proxy server to use 
 *                   e.g. proxy.mydomain.com
 * @param serverPort the port number to use. e.g. 8080
 */
void       tp_SetProxy          (tunepimp_t o, char *serverAddr, short serverPort);

int tp_GetNumSupportedExtensions(tunepimp_t o);
void tp_GetSupportedExtensions(tunepimp_t o, char extensions[][32]);
void tp_SetAutoFileLookup(tunepimp_t o, int enable);
void tp_SetNotifyCallback(tunepimp_t o, tp_notify_callback callback);
void tp_SetStatusCallback(tunepimp_t o, tp_status_callback callback);
void tp_GetError(tunepimp_t o, char *error, int maxLen);

/**
 * Enable debug out to stdout by sending a non-zero value to this
 * function. 
 * @param o the tunepimp_t object returned from tp_New
 * @param debug whether or not to enable debug (non zero enables debug output)
 */
void       tp_SetDebug        (tunepimp_t o, int debug);

/* Returns a fileId number that can be used to reference the tracks later */
int       tp_Add                   (tunepimp_t o, const char *fileName);
int       tp_AddDir                (tunepimp_t o, const char *dirPath);
void      tp_Remove                (tunepimp_t o, int fileId);
int       tp_GetNumFiles           (tunepimp_t o);
int       tp_GetNumUnsubmitted     (tunepimp_t o);
int       tp_GetNumFileIds         (tunepimp_t o);
void      tp_GetFileIds            (tunepimp_t o, int *ids);
track_t   tp_GetTrack             (tunepimp_t o, int fileId);
void      tp_ReleaseTrack          (tunepimp_t o, track_t track);
void      tp_Wake                  (tunepimp_t o, track_t track);
TPError   tp_SelectResult          (tunepimp_t o, track_t track, int resultIndex);
void      tp_Misidentified         (tunepimp_t o, int fileId);
void      tp_IdentifyAgain         (tunepimp_t o, int fileId);
/* fileIds could be zero. If that, numFileIds is ignored */
int      tp_WriteTags             (tunepimp_t o, int *fileIds, int numFileIds);
TPError   tp_SubmitTRMs            (tunepimp_t o);
void      tp_SetRenameFiles          (tunepimp_t o, int rename);
void      tp_SetMoveFiles            (tunepimp_t o, int move);
void      tp_SetWriteID3v1           (tunepimp_t o, int writeID3v1);
void      tp_SetWriteID3v2           (tunepimp_t o, int writeID3v2);
void      tp_SetClearTags            (tunepimp_t o, int clearTags);
void      tp_SetFileMask             (tunepimp_t o, const char *fileMask);
void      tp_SetVariousFileMask      (tunepimp_t o, const char *variousFileMask);
void      tp_SetAllowedFileCharacters(tunepimp_t o, const char *allowedFileCharacters);
void      tp_SetDestDir              (tunepimp_t o, const char *destDir);
void      tp_SetTopSrcDir            (tunepimp_t o, const char *topSrcDir);
void      tp_SetTRMCollisionThreshold(tunepimp_t o, int trmThreshold);
void      tp_SetAutoSaveThreshold    (tunepimp_t o, int autoSaveThreshold);

#ifdef WIN32
/**
 * WINDOWS ONLY: This function must be called to initialize the WinSock
 * TCP/IP stack in windows. If your application does not utilize any
 * WinSock functions, you must call this function before you can call
 * tp_Query(). Before your application shuts down, you must call tp_WSAStop().
 * If you already call WSAInit from your own application,
 * you do not need to call this function.
 * @see tp_WSAStop.
 * @param o the tunepimp_t object returned from tp_New()
 */
void      tp_WSAInit           (tunepimp_t o);

/**
 * WINDOWS ONLY: Call this function when your application shuts down. Only
 * call this function if you called tp_WSAInit().
 * @see tp_WSAInit
 * @param o the tunepimp_t object returned from tp_New()
 * @param
 */
void      tp_WSAStop           (tunepimp_t o);
#endif 

/* --------------------------------------------------------------------------
 * Track Interface
 * --------------------------------------------------------------------------*/

TPFileStatus tr_GetStatus      (track_t t);
void       tr_SetStatus        (track_t t, const TPFileStatus status);
void       tr_GetFileName      (track_t t, char *fileName, int maxLen);
void       tr_GetTRM           (track_t t, char *trm, int maxLen);
void       tr_GetLocalMetadata (track_t t, metadata_t *mdata);
void       tr_SetLocalMetadata (track_t t, const metadata_t *mdata);
void       tr_GetServerMetadata(track_t t, metadata_t *mdata);
void       tr_SetServerMetadata(track_t t, const metadata_t *mdata);
void       tr_GetError         (track_t t, char *error, int maxLen);
int        tr_GetSimilarity    (track_t t);
int        tr_HasChanged       (track_t t);
int        tr_GetNumResults    (track_t t);
void       tr_GetResults       (track_t t, TPResultType *type, 
                                result_t *results, int numResults);
void       tr_Lock             (track_t t);
void       tr_Unlock           (track_t t);

/* --------------------------------------------------------------------------
 * Metadata Interface
 * --------------------------------------------------------------------------*/


metadata_t   *md_New                   (void);
void          md_Delete                (metadata_t *);
void          md_Clear                 (metadata_t *);
TPAlbumStatus md_ConvertToAlbumStatus  (const char *albumStatus);
TPAlbumType   md_ConvertToAlbumType    (const char *albumType);
void          md_ConvertFromAlbumStatus(TPAlbumStatus status, char *albumStatus, int maxLen);
void          md_ConvertFromAlbumType  (TPAlbumType type, char *albumType, int maxLen);

/* --------------------------------------------------------------------------
 * Result Interface
 * --------------------------------------------------------------------------*/


void          rs_Delete                (TPResultType *type, result_t *result);


#ifdef __cplusplus
}
#endif

#endif 
